{% load djblets_forms djblets_utils %}

{% for fieldset_title, fieldset in form|get_fieldsets %}
{%  include "djblets_forms/admin/form_fieldset.html" %}
{% empty %}
  <fieldset class="module aligned">
{%  for field in form %}
{%   include "djblets_forms/admin/form_field.html" %}
{%  endfor %}
  </fieldset>
{% endfor %}
