/***************************************************************************
 * Related object autocomplete
 ***************************************************************************/
.related-object-options,
.related-object-selected {
  font-size: 11px;
}
.related-object-options img,
.related-object-selected img {
  width: 20px;
  height: 20px;
  border-radius: 10px;
  vertical-align: top;
}
.related-object-options span,
.related-object-selected span {
  line-height: 20px;
  vertical-align: top;
}
.related-object-options .title,
.related-object-selected .title {
  display: inline-block;
  min-width: 20em;
  font-size: 11px;
}
.related-object-options .description,
.related-object-selected .description {
  display: inline-block;
  font-size: 9.9px;
  padding: 0;
}
.related-object-selector {
  vertical-align: middle;
}
.related-object-selector .related-object-selected {
  border: 1px solid #b8b8b8;
  border-radius: 3px;
  box-shadow: inset 0 1px 0 rgba(0, 0, 0, 0.05), 0 1px 0 rgba(255, 255, 255, 0.8);
  list-style: none;
  margin: 0.5em 0 0 0;
  max-height: 20em;
  min-height: 3em;
  overflow-y: auto;
  padding: 0;
}
.related-object-selector .related-object-selected li {
  cursor: default;
  padding: 5px 8px;
  vertical-align: middle;
}
.related-object-selector .related-object-selected li div {
  display: inline-block;
}
.related-object-selector .related-object-selected li .fa-close {
  color: #ccc;
  cursor: pointer;
  float: right;
}
.related-object-selector .related-object-selected li .fa-close:hover {
  color: black;
}
.related-object-selector .selectize-input {
  font-size: 11px;
  padding: 4px;
}
.related-object-selector .selectize-input::after {
  right: 10px;
}
.related-object-selector .selectize-input input {
  width: 100% !important;
  /* To overwrite the measureString() return value,
      which seems to  be broken in selectize-0.12.1 */
}
/*# sourceMappingURL=related-object-selector.css.map */