/** A spinner component. */
/**
 * A spinner.
 *
 * Spinners represent an in-progress activity, such as new content being
 * loaded or a request being sent to a server.
 *
 * The default spinner uses a black rotating curve. Consumers can use
 * ``#djblets-ns-ui.spinners.set-theme(light)`` on a selector to switch to
 * a white spinner, for darker backgrounds.
 *
 * Accessibility Notes:
 *     When using the spinner to represent content loading in a parent element,
 *     it's best to set ``aria-busy="true"`` on that element in order to inform
 *     screen readers that the content is still loading. It's very important to
 *     remove this attribute once the load has completed.
 *
 *     This should be used along with ``aria-live=`` to indicate that the
 *     element will be updated.
 *
 *     Alternatively, you might want to use ``aria-hidden="false"`` on this, if
 *     it should not be read or seen by the screen reader, or
 *     ``aria-label="..."`` if it should (and does not otherwise have
 *     accompanying text).
 *
 * Structure:
 *     <div class="djblets-o-spinner"></div>
 */
.djblets-o-spinner {
  background-repeat: no-repeat;
  display: inline-block;
  width: 1em;
  height: 1em;
  vertical-align: middle;
  background-image: url("data:image/svg+xml,%3Csvg%20width%3D%22100%25%22%20height%3D%22100%25%22%20viewBox%3D%220%200%2016%2016%22%20version%3D%221.1%22%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20style%3D%22stroke-linecap%3Around%3Bstroke-linejoin%3Around%3Bstroke-miterlimit%3A1.5%3B%22%3E%0A%20%3Cpath%20d%3D%22M11.514%2C14.035c-1.032%2C0.603%20-2.233%2C0.948%20-3.514%2C0.948c-3.854%2C0%20-6.983%2C-3.129%20-6.983%2C-6.983c0%2C-3.854%203.129%2C-6.983%206.983%2C-6.983%22%20style%3D%22fill%3Anone%3Bstroke%3A%23000%3Bstroke-width%3A2px%3B%22%2F%3E%0A%20%3CanimateTransform%0A%20%20%20attributeName%3D%22transform%22%0A%20%20%20type%3D%22rotate%22%0A%20%20%20from%3D%220%200%200%22%0A%20%20%20to%3D%22360%200%200%22%0A%20%20%20dur%3D%221.0s%22%0A%20%20%20repeatCount%3D%22indefinite%22%2F%3E%0A%3C%2Fsvg%3E%0A");
}
/*# sourceMappingURL=spinner.css.map */