{% spaceless %}
<span class="avatar djblets-o-avatar djblets-o-avatar-fallback"
      title="{{user.username}}"
      style="background: {{bg}}; display: inline-block; width: {{size}}px; line-height: {{size}}px; font-size: {{font_size}}; text-align: center;">
 {{text}}
</span>
{% endspaceless %}
