@markdown-table-header-border-color: #000000;
@markdown-table-border-color: #bbbbbb;

/* Font size for RelatedObjectWidget */
@widget_font_size: 11px;

/*
 * These are obsolete now that LESS compilation includes autoprefixer. They're
 * left here for compatibility with third-parties, but should be considered
 * deprecated in favor of just using the standard CSS rules.
 */
.border-radius(...) { border-radius: @arguments; }
.pre-wrap() { white-space: pre-wrap; }
